@@ -23,8 +23,16 @@ from mlserver.model import MLModel
 from mlserver.utils import get_model_uri
 from mlserver.handlers import custom_handler
 from mlserver.errors import InferenceError
+from mlserver.settings import ModelParameters
+from mlserver.logging import logger
 
-from .encoding import to_outputs
+from .codecs import TensorDictCodec
+from .metadata import (
+    to_metadata_tensors,
+    to_model_content_type,
+    DefaultInputPrefix,
+    DefaultOutputPrefix,
+)
 
 
 class MLflowRuntime(MLModel):
@@ -33,6 +41,19 @@ class MLflowRuntime(MLModel):
     models persisted with `joblib`.
     """
 
+    # TODO: Decouple from REST
+    @custom_handler(rest_path="/ping", rest_method="GET")
+    async def ping(self, request: Request) -> str:
+        """
+        This custom handler is meant to mimic the behaviour of the existing
+        health endpoint in MLflow's local dev server.
+        For details about its implementation, please consult the original
+        implementation in the MLflow repository:
+
+            https://github.com/mlflow/mlflow/blob/master/mlflow/pyfunc/scoring_server/__init__.py
+        """
+        return "\n"
+
     # TODO: Decouple from REST
     @custom_handler(rest_path="/invocations")
     async def invocations(self, request: Request) -> Response:
@@ -94,19 +115,46 @@ class MLflowRuntime(MLModel):
         # TODO: Log info message
         model_uri = await get_model_uri(self._settings)
         self._model = mlflow.pyfunc.load_model(model_uri)
+
         self._input_schema = self._model.metadata.get_input_schema()
+        self._signature = self._model.metadata.signature
+        self._sync_metadata()
 
         self.ready = True
         return self.ready
 
+    def _sync_metadata(self) -> None:
+        # Update metadata from model signature (if present)
+        if self._signature is None:
+            return
+
+        if self.inputs:
+            logger.warning("Overwriting existing inputs metadata with model signature")
+
+        self.inputs = to_metadata_tensors(
+            schema=self._signature.inputs, prefix=DefaultInputPrefix
+        )
+
+        if self.outputs:
+            logger.warning("Overwriting existing outputs metadata with model signature")
+
+        self.outputs = to_metadata_tensors(
+            schema=self._signature.outputs, prefix=DefaultOutputPrefix
+        )
+
+        if not self._settings.parameters:
+            self._settings.parameters = ModelParameters()
+
+        if self._settings.parameters.content_type:
+            logger.warning(
+                "Overwriting existing request-level content type with model signature"
+            )
+
+        self._settings.parameters.content_type = to_model_content_type(
+            schema=self._signature.inputs
+        )
+
     async def predict(self, payload: InferenceRequest) -> InferenceResponse:
         decoded_payload = self.decode_request(payload)
-
-        # TODO: Can `output` be a dictionary of tensors?
         model_output = self._model.predict(decoded_payload)
-
-        return InferenceResponse(
-            model_name=self.name,
-            model_version=self.version,
-            outputs=to_outputs(model_output),
-        )
+        return self.encode_response(model_output, default_codec=TensorDictCodec)
